@@ -21,14 +21,13 @@
 #include "screencaststream.h"
 #include "waylandintegration.h"
 
+#include <limits.h>
 #include <math.h>
 #include <sys/mman.h>
 #include <stdio.h>
 
 #include <QLoggingCategory>
-#include <QTimer>
 #include <QSize>
-#include <QSocketNotifier>
 
 Q_LOGGING_CATEGORY(XdgDesktopPortalKdeScreenCastStream, "xdp-kde-screencast-stream")
 
@@ -57,7 +56,7 @@ static int greatestCommonDivisor(int a, int b)
         b = temp % b;
     }
 
-    return ABS(a);
+    return qAbs(a);
 }
 
 static PwFraction pipewireFractionFromDouble(double src)
@@ -69,13 +68,13 @@ static PwFraction pipewireFractionFromDouble(double src)
     int64_t N2, D2;              /* numerator, denominator of previous approx */
     int i;
     int gcd;
-    gboolean negative = FALSE;
+    bool negative = false;
 
     /* initialize fraction being converted */
     F = src;
     if (F < 0.0) {
         F = -F;
-        negative = TRUE;
+        negative = true;
     }
 
     V = F;
@@ -89,7 +88,7 @@ static PwFraction pipewireFractionFromDouble(double src)
 
     for (i = 0; i < MAX_TERMS; ++i) {
         /* get next term */
-        A = (gint) F;               /* no floor() needed, F is always >= 0 */
+        A = (int) F;               /* no floor() needed, F is always >= 0 */
         /* get new divisor */
         F = F - A;
 
@@ -98,7 +97,7 @@ static PwFraction pipewireFractionFromDouble(double src)
         D2 = D1 * A + D2;
 
         /* guard against overflow */
-        if (N2 > G_MAXINT || D2 > G_MAXINT)
+        if (N2 > INT_MAX || D2 > INT_MAX)
             break;
 
         N = N2;
@@ -111,7 +110,7 @@ static PwFraction pipewireFractionFromDouble(double src)
         D1 = D;
 
         /* quit if dividing by zero or close enough to target */
-        if (F < MIN_DIVISOR || fabs (V - ((gdouble) N) / D) < MAX_ERROR)
+        if (F < MIN_DIVISOR || fabs (V - ((double) N) / D) < MAX_ERROR)
             break;
 
         /* Take reciprocal */
@@ -120,7 +119,7 @@ static PwFraction pipewireFractionFromDouble(double src)
 
     /* fix for overflow */
     if (D == 0) {
-        N = G_MAXINT;
+        N = INT_MAX;
         D = 1;
     }
 
@@ -142,7 +141,7 @@ static PwFraction pipewireFractionFromDouble(double src)
     return fraction;
 }
 
-static void onStateChanged(void *data, pw_remote_state old, pw_remote_state state, const char *error)
+void ScreenCastStream::onStateChanged(void *data, pw_remote_state old, pw_remote_state state, const char *error)
 {
     Q_UNUSED(old);
 
@@ -166,7 +165,7 @@ static void onStateChanged(void *data, pw_remote_state old, pw_remote_state stat
     }
 }
 
-static void onStreamStateChanged(void *data, pw_stream_state old, pw_stream_state state, const char *error_message)
+void ScreenCastStream::onStreamStateChanged(void *data, pw_stream_state old, pw_stream_state state, const char *error_message)
 {
     Q_UNUSED(old)
 
@@ -194,7 +193,7 @@ static void onStreamStateChanged(void *data, pw_stream_state old, pw_stream_stat
     }
 }
 
-static void onStreamFormatChanged(void *data, const struct spa_pod *format)
+void ScreenCastStream::onStreamFormatChanged(void *data, const struct spa_pod *format)
 {
     qCDebug(XdgDesktopPortalKdeScreenCastStream) << "Stream format changed";
 
@@ -239,36 +238,28 @@ static void onStreamFormatChanged(void *data, const struct spa_pod *format)
                                                    ":", pw->pwCoreType->param_buffers.buffers, "iru", 16, PROP_RANGE (2, 16),
                                                    ":", pw->pwCoreType->param_buffers.align, "i", 16);
 #endif
-    pw_stream_finish_format (pw->pwStream, 0,
-                             params, G_N_ELEMENTS (params));
+    pw_stream_finish_format (pw->pwStream, 0, params, 1);
 }
 
-static const struct pw_remote_events pwRemoteEvents = {
-    .version = PW_VERSION_REMOTE_EVENTS,
-    .destroy = nullptr,
-    .info_changed = nullptr,
-    .sync_reply = nullptr,
-    .state_changed = onStateChanged,
-};
-
-static const struct pw_stream_events pwStreamEvents = {
-    .version = PW_VERSION_STREAM_EVENTS,
-    .destroy = nullptr,
-    .state_changed = onStreamStateChanged,
-    .format_changed = onStreamFormatChanged,
-    .add_buffer = nullptr,
-    .remove_buffer = nullptr,
-    .process = nullptr,
-};
-
 ScreenCastStream::ScreenCastStream(const QSize &resolution, QObject *parent)
     : QObject(parent)
     , resolution(resolution)
 {
+    // initialize event handlers, remote end and stream-related
+    pwRemoteEvents.version = PW_VERSION_REMOTE_EVENTS;
+    pwRemoteEvents.state_changed = &onStateChanged;
+
+    pwStreamEvents.version = PW_VERSION_STREAM_EVENTS;
+    pwStreamEvents.state_changed = &onStreamStateChanged;
+    pwStreamEvents.format_changed = &onStreamFormatChanged;
 }
 
 ScreenCastStream::~ScreenCastStream()
 {
+    if (pwMainLoop) {
+        pw_thread_loop_stop(pwMainLoop);
+    }
+
 #if !PW_CHECK_VERSION(0, 2, 9)
     if (pwType) {
         delete pwType;
@@ -283,12 +274,14 @@ ScreenCastStream::~ScreenCastStream()
         pw_remote_destroy(pwRemote);
     }
 
-    if (pwCore) {
+    if (pwCore)
         pw_core_destroy(pwCore);
+
+    if (pwMainLoop) {
+        pw_thread_loop_destroy(pwMainLoop);
     }
 
     if (pwLoop) {
-        pw_loop_leave(pwLoop);
         pw_loop_destroy(pwLoop);
     }
 }
@@ -298,8 +291,7 @@ void ScreenCastStream::init()
     pw_init(nullptr, nullptr);
 
     pwLoop = pw_loop_new(nullptr);
-    socketNotifier.reset(new QSocketNotifier(pw_loop_get_fd(pwLoop), QSocketNotifier::Read));
-    connect(socketNotifier.data(), &QSocketNotifier::activated, this, &ScreenCastStream::processPipewireEvents);
+    pwMainLoop = pw_thread_loop_new(pwLoop, "pipewire-main-loop");
 
     pwCore = pw_core_new(pwLoop, nullptr);
 #if !PW_CHECK_VERSION(0, 2, 9)
@@ -314,6 +306,10 @@ void ScreenCastStream::init()
     pw_remote_add_listener(pwRemote, &remoteListener, &pwRemoteEvents, this);
 
     pw_remote_connect(pwRemote);
+
+    if (pw_thread_loop_start(pwMainLoop) < 0) {
+        qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to start main PipeWire loop";
+    }
 }
 
 uint ScreenCastStream::framerate()
@@ -387,9 +383,9 @@ bool ScreenCastStream::createStream()
     auto flags = static_cast<pw_stream_flags>(PW_STREAM_FLAG_DRIVER | PW_STREAM_FLAG_MAP_BUFFERS);
 
 #if PW_CHECK_VERSION(0, 2, 9)
-    if (pw_stream_connect(pwStream, PW_DIRECTION_OUTPUT, 0, flags, params, G_N_ELEMENTS(&params)) != 0) {
+    if (pw_stream_connect(pwStream, PW_DIRECTION_OUTPUT, 0, flags, params, 1) != 0) {
 #else
-    if (pw_stream_connect(pwStream, PW_DIRECTION_OUTPUT, nullptr, flags, params, G_N_ELEMENTS(&params)) != 0) {
+    if (pw_stream_connect(pwStream, PW_DIRECTION_OUTPUT, nullptr, flags, params, 1) != 0) {
 #endif
         qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Could not connect to stream";
         return false;
@@ -444,11 +440,3 @@ void ScreenCastStream::initializePwTypes()
     spa_type_video_format_map (map, &pwType->video_format);
 }
 #endif
-
-void ScreenCastStream::processPipewireEvents()
-{
-    int result = pw_loop_iterate(pwLoop, 0);
-    if (result < 0) {
-        qCWarning(XdgDesktopPortalKdeScreenCastStream) << "Failed to iterate over pipewire loop: " << spa_strerror(result);
-    }
-}
